<!-- THIS IS AN AUTOGENERATED FILE. DO NOT EDIT THIS FILE DIRECTLY. -->
<!-- This file is created by the `yarn generate-readme` script. -->

# AsyncSelectField

## Description

A controlled input component with validation states and a label getting a selection from an asynchronously loaded list from the user.

## Installation

```
yarn add @commercetools-uikit/async-select-field
```

```
npm --save install @commercetools-uikit/async-select-field
```

Additionally install the peer dependencies (if not present)

```
yarn add react
```

```
npm --save install react
```

## Usage

```jsx
import AsyncSelectField from '@commercetools-uikit/async-select-field';

const Example = () => (
  <AsyncSelectField
    title="State"
    value={{ value: 'ready', label: 'Ready' }}
    loadOptions={
      (/* inputValue */) => {
        // async fetch logic
      }
    }
    onChange={(event) => alert(event.target.value)}
  />
);

export default Example;
```

## Properties

| Props                      | Type                                                                                                  | Required | Default   | Description                                                                                                                                                                                                                                                           |
| -------------------------- | ----------------------------------------------------------------------------------------------------- | :------: | --------- | --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `id`                       | `AsyncProps['inputId']`                                                                               |          |           | Used as HTML id property. An id is auto-generated when it is not specified.&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                           |
| `horizontalConstraint`     | `union`<br/>Possible values:<br/>`, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 'scale', 'auto'` |          | `'scale'` | Horizontal size limit of the input fields.                                                                                                                                                                                                                            |
| `errors`                   | `Record`                                                                                              |          |           | A map of errors. Error messages for known errors are rendered automatically.&#xA;<br />&#xA;Unknown errors will be forwarded to `renderError`                                                                                                                         |
| `renderError`              | `Function`<br/>[See signature.](#signature-rendererror)                                               |          |           | Called with custom errors. This function can return a message which will be wrapped in an ErrorMessage. It can also return null to show no error.                                                                                                                     |
| `warnings`                 | `Record`                                                                                              |          |           | A map of warnings. Warning messages for known warnings are rendered automatically.&#xA;<br />&#xA;Unknown warnings will be forwarded to `renderWarning`                                                                                                               |
| `renderWarning`            | `Function`<br/>[See signature.](#signature-renderwarning)                                             |          |           | Called with custom warnings. This function can return a message which will be wrapped in an WarningMessage. It can also return null to show no warning.                                                                                                               |
| `isRequired`               | `boolean`                                                                                             |          |           | Indicates if the value is required. Shows an the "required asterisk" if so.                                                                                                                                                                                           |
| `touched`                  | `union`<br/>Possible values:<br/>`boolean[] , boolean`                                                |          |           | Indicates whether the field was touched. Errors will only be shown when the field was touched.                                                                                                                                                                        |
| `aria-label`               | `AsyncProps['aria-label']`                                                                            |          |           | Aria label (for assistive tech)&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                                       |
| `aria-labelledby`          | `AsyncProps['aria-labelledby']`                                                                       |          |           | HTML ID of an element that should be used as the label (for assistive tech)&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                           |
| `isAutofocussed`           | `boolean`                                                                                             |          |           | Focus the control when it is mounted                                                                                                                                                                                                                                  |
| `backspaceRemovesValue`    | `AsyncProps['backspaceRemovesValue']`                                                                 |          |           | Remove the currently focused option when the user presses backspace&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                   |
| `components`               | `AsyncProps['components']`                                                                            |          |           | Map of components to overwrite the default ones, see [what components you can override](https://react-select.com/components)&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                          |
| `controlShouldRenderValue` | `AsyncProps['controlShouldRenderValue']`                                                              |          | `true`    | Control whether the selected values should be rendered in the control&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                 |
| `filterOption`             | `AsyncProps['filterOption']`                                                                          |          |           | Custom method to filter whether an option should be displayed in the menu&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                             |
| `containerId`              | `AsyncProps['id']`                                                                                    |          |           | The id to set on the SelectContainer component&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                        |
| `isClearable`              | `AsyncProps['isClearable']`                                                                           |          |           | Is the select value clearable&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                                         |
| `isCondensed`              | `boolean`                                                                                             |          |           | Use this property to reduce the paddings of the component for a ui compact variant                                                                                                                                                                                    |
| `isDisabled`               | `AsyncProps['isDisabled']`                                                                            |          |           | Is the select disabled&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                                                |
| `isReadOnly`               | `boolean`                                                                                             |          |           | Is the select read-only                                                                                                                                                                                                                                               |
| `isOptionDisabled`         | `AsyncProps['isOptionDisabled']`                                                                      |          |           | Override the built-in logic to detect whether an option is disabled&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                   |
| `isMulti`                  | `AsyncProps['isMulti']`                                                                               |          |           | Support multiple selected options&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                                     |
| `isSearchable`             | `AsyncProps['isSearchable']`                                                                          |          |           | Whether to enable search functionality&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                                |
| `hasWarning`               | `boolean`                                                                                             |          |           | Indicates the input field has a warning&#xA;@deprecated Please use the `warnings` prop instead so users know the reason why the field is in warning state.                                                                                                            |
| `maxMenuHeight`            | `AsyncProps['maxMenuHeight']`                                                                         |          |           | Maximum height of the menu before scrolling&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                           |
| `menuPortalTarget`         | `AsyncProps['menuPortalTarget']`                                                                      |          |           | Dom element to portal the select menu to&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                              |
| `menuPortalZIndex`         | `number`                                                                                              |          |           | z-index value for the menu portal&#xA;<br>&#xA;Use in conjunction with `menuPortalTarget`                                                                                                                                                                             |
| `menuShouldBlockScroll`    | `AsyncProps['menuShouldBlockScroll']`                                                                 |          |           | whether the menu should block scroll while open&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                       |
| `name`                     | `AsyncProps['name']`                                                                                  |          |           | Name of the HTML Input (optional - without this, no input will be rendered)&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                           |
| `noOptionsMessage`         | `AsyncProps['noOptionsMessage']`                                                                      |          |           | Can be used to render a custom value when there are no options (either because of no search results, or all options have been used, or there were none in the first place).&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)           |
| `onBlur`                   | `Function`<br/>[See signature.](#signature-onblur)                                                    |          |           | Handle blur events on the control                                                                                                                                                                                                                                     |
| `onChange`                 | `Function`<br/>[See signature.](#signature-onchange)                                                  |          |           | Called with a fake event when value changes.&#xA;<br />&#xA;The event's `target.name` will be the name supplied in props. The event's `target.value` will hold the value. The value will be the selected option, or an array of options in case `isMulti` is `true`.  |
| `onFocus`                  | `FocusEventHandler`                                                                                   |          |           | Handle focus events on the control                                                                                                                                                                                                                                    |
| `onInputChange`            | `AsyncProps['onInputChange']`                                                                         |          |           | Handle change events on the input&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                                     |
| `placeholder`              | `AsyncProps['placeholder']`                                                                           |          |           | Placeholder text for the select value&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                                 |
| `loadingMessage`           | `union`<br/>Possible values:<br/>`string , (() => string)`                                            |          |           | loading message shown while the options are being loaded                                                                                                                                                                                                              |
| `tabIndex`                 | `AsyncProps['tabIndex']`                                                                              |          |           | Sets the tabIndex attribute on the input&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                                              |
| `tabSelectsValue`          | `AsyncProps['tabSelectsValue']`                                                                       |          |           | Select the currently focused option when the user presses tab&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                         |
| `value`                    | `AsyncProps['value']`                                                                                 |          |           | The value of the select; reflected by the selected option&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                                                             |
| `showOptionGroupDivider`   | `boolean`                                                                                             |          |           | Determines if option groups will be separated by a divider                                                                                                                                                                                                            |
| `defaultOptions`           | `union`<br/>Possible values:<br/>`OptionsOrGroups<unknown, GroupBase<unknown>> , boolean`             |          |           | The default set of options to show before the user starts searching.&#xA;<br />&#xA;When set to `true`, the results for `loadOptions('')` will be autoloaded.                                                                                                         |
| `loadOptions`              | `AsyncProps['loadOptions']`                                                                           |    ✅    |           | Function that returns a promise, which is the set of options to be used once the promise resolves.&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                                                    |
| `cacheOptions`             | `AsyncProps['cacheOptions']`                                                                          |          |           | If cacheOptions is truthy, then the loaded data will be cached. The cache will remain until cacheOptions changes value.&#xA;<br>&#xA;[Props from React select was used](https://react-select.com/props)                                                               |
| `title`                    | `union`<br/>Possible values:<br/>`string , ReactNode`                                                 |    ✅    |           | Title of the label                                                                                                                                                                                                                                                    |
| `hint`                     | `union`<br/>Possible values:<br/>`string , ReactNode`                                                 |          |           | Hint for the label. Provides a supplementary but important information regarding the behaviour of the input (e.g warn about uniqueness of a field, when it can only be set once), whereas `description` can describe it in more depth. Can also receive a `hintIcon`. |
| `description`              | `union`<br/>Possible values:<br/>`string , ReactNode`                                                 |          |           | Provides a description for the title.                                                                                                                                                                                                                                 |
| `onInfoButtonClick`        | `Function`<br/>[See signature.](#signature-oninfobuttonclick)                                         |          |           | Function called when info button is pressed.&#xA;<br />&#xA;Info button will only be visible when this prop is passed.                                                                                                                                                |
| `hintIcon`                 | `ReactElement`                                                                                        |          |           | Icon to be displayed beside the hint text.&#xA;<br />&#xA;Will only get rendered when `hint` is passed as well.                                                                                                                                                       |
| `badge`                    | `ReactNode`                                                                                           |          |           | Badge to be displayed beside the label.&#xA;<br />&#xA;Might be used to display additional information about the content of the field (E.g verified email)                                                                                                            |
| `iconLeft`                 | `ReactNode`                                                                                           |          |           | Icon to display on the left of the placeholder text and selected value. Has no effect when isMulti is enabled.                                                                                                                                                        |

## Signatures

### Signature `renderError`

```ts
(key: string, error?: boolean) => ReactNode;
```

### Signature `renderWarning`

```ts
(key: string, warning?: boolean) => ReactNode;
```

### Signature `onBlur`

```ts
(event: TCustomEvent) => void
```

### Signature `onChange`

```ts
(event: TCustomEvent, info: ActionMeta<unknown>) => void
```

### Signature `onInfoButtonClick`

```ts
() => void
```

## `data-*` props

The component further forwards all `data-` attributes to the underlying `AsyncSelectInput` component.

This field is built on top of [`react-select`](https://github.com/JedWatson/react-select) v2.
It supports mostly same properties as `react-select`. Behaviour for some props was changed, and support for others was dropped.

In case you need one of the currently excluded props, feel free to open a PR adding them.

## `errors`

This object is a key-value map. The `renderError` prop will be called for each entry with the key and the value. The return value will be rendered inside an `ErrorMessage` component underneath the input.

The `AsyncSelectField` supports some errors out of the box. Return `undefined` from `renderError` for these and the default errors will be shown instead. This prevents consumers from having to reimplement the same error messages for known errors while still keeping the flexibility of showing custom error messages for them.

When the `key` is known, and when the value is truthy, and when `renderError` returned `undefined` for that error entry, then the `AsyncSelectField` will render an appropriate error automatically.

Known error keys are:

- `missing`: tells the user that this field is required

## Static methods

### `AsyncSelectField.toFieldErrors`

Use this function to convert the Formik `errors` object type to our custom field errors type. This is primarily useful when using TypeScript.

```ts
type FormValues = {
  myField: string;
};

<AsyncSelectField
  // ...
  name="my-field"
  errors={AsyncSelectField.toFieldErrors<FormValues>(formik.errors).myField}
/>;
```
